@@ -25,6 +25,7 @@ coef.panelmodel <- function(object,...){
 
 plmformat <- function(object,names,rnames,
                       df.residual,model.name,pdim,pmodel,indexes,cl,...){
+  interc <- pmodel$has.intercept
   coefficients <- coefficients(object)
   residuals <- residuals(object)
   fitted.values <- fitted.values(object)
@@ -35,13 +36,19 @@ plmformat <- function(object,names,rnames,
   else{
     vcov <- vcov(object)
   }
-  cl$model.name=model.name
-  model <- object$model
-  colnames(model)[1] <- as.character(formula(object))[[2]]
-  rownames(model) <- names(residuals) <- names(fitted.values) <- rnames
-  names(coefficients) <- rownames(vcov) <- colnames(vcov) <- names
+  cl$model.name <- model.name
+  response.name <- deparse(pmodel$formula[[2]])
+  mymodel <- object$model
+  mymodel <- object$model[[2]]
+  mymodel <- data.frame(object$model[[1]],mymodel)
+  if (model.name == "random" && interc == TRUE) names(mymodel) <- c(response.name,c("(intercept)",names))
+  else names(mymodel) <- c(response.name,names)
+  attr(mymodel,"terms") <- terms(pmodel$formula)
+  rownames(mymodel) <- names(residuals) <- names(fitted.values) <- rnames
+  if (interc && model.name != "within") coefnames <- c("(intercept)",names) else coefnames <- names
+  names(coefficients) <- rownames(vcov) <- colnames(vcov) <- coefnames
   object <- list(coefficients=coefficients,residuals=residuals,
-                 fitted.values=fitted.values,vcov=vcov,df.residual=df.residual,model=model,
+                 fitted.values=fitted.values,vcov=vcov,df.residual=df.residual,model=mymodel,
                  call=cl)
   object <- structure(object,pdim=pdim,pmodel=pmodel,indexes=indexes,class=c("plm","panelmodel"))
 }
@@ -84,3 +91,29 @@ print.theta <- function(x,digits){
     }
   }
 }
+
+## model.matrix.panelmodel <- function(object, ...){
+##   data <- model.frame(object, ...)
+##   form <- attr(data,"terms")
+##   m <- model.matrix(form,data)
+##   if (attr(object,"pmodel")$model.name == "random" && attr(object,"pmodel")$has.intercept == TRUE){
+##     m[,1] <- data[["(intercept)"]]
+##   }
+##   if (attr(object,"pmodel")$model.name == "within" && attr(object,"pmodel")$has.intercept == TRUE){
+##     m <- m[, -1, drop = FALSE]
+##   }
+##   m
+## }
+
+model.matrix.panelmodel <- function(object, ...){
+  m <- as.matrix(model.frame(object, ...)[, -1, drop = FALSE])
+#  if (attr(object,"pmodel")$model.name == "random" && attr(object,"pmodel")$has.intercept == TRUE){
+#    m[,1] <- data[["(intercept)"]]
+#  }
+  if (attr(object,"pmodel")$model.name %in% c("between","pooling") && attr(object,"pmodel")$has.intercept == TRUE){
+    m <- cbind('(intercept)'=1,m)
+  }
+  m
+}
+
+
